/* $Id: download.hh,v 1.4 2001/11/19 00:04:50 richard Exp $ -*- C++ -*-
  __   _
  |_) /|  Copyright (C) 2001 Richard Atterer
  | \/|  <richard@atterer.net>
   ` 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2. See
  the file COPYING for details.

  Download data from URL, write to output function, report on progress

*/

#ifndef DOWNLOAD_HH
#define DOWNLOAD_HH

#ifndef INLINE
#  ifdef NOINLINE
#    define INLINE
#    else
#    define INLINE inline
#  endif
#endif

#include <string>
namespace std { }
using namespace std;

//#include <download.fh>
//______________________________________________________________________

/** Class containing the state of a download. libwww must be properly
    initialised libwww before usage (this happens in glibwww-init.cc). */
class Download {
public:
  /// Class for outputting progress reports and data
  class Output;
  inline Download(const string& uri);
  ~Download();
  inline const string& uri() const;
  inline void setOutput(Output* o);
private:
  string uriVal;
  Output* output;
};
//______________________________________________________________________

/** A derived class of Output must be supplied by the user of
    Download, to take care of the downloaded data and to output progress
    reports. */
class Download::Output {
public:
  virtual ~Output() { }
  /** Called as soon as the size of the downloaded data is known. May
      not be called at all if the size is unknown. */
  virtual void dataSize(uint64) { }
  /** Called during download whenever data arrives, with the data that
      just arrived. You can write the data to a file, copy it away
      etc. totalSize is the offset into the downloaded data - useful
      for "x% done" messages. */
  virtual void newData(void* /*data*/, size_t /*size*/, uint64 /*totalSize*/) { }
  /** Short progress message ("Looking up host", "Connecting" etc). As
      a special case, an empty string indicates that the download
      proper has begun. */
  virtual void info(const string& /*message*/) { }
  /** Short progress message ("Timeout", "404 not found") which means
      that the download has failed. */
  virtual void error(const string& /*message*/) { }
};
//______________________________________________________________________

Download::Download(const string& uri) : uriVal(uri), output(0) { }

const string& Download::uri() const { return uriVal; }

void Download::setOutput(Output* o) { output = o; }

#ifndef NOINLINE
//#  include <download.ih> /* NOINLINE */
#endif

#endif
