/* $Id: joblist.cc,v 1.8 2001/12/01 23:01:31 richard Exp $ -*- C++ -*-
  __   _
  |_) /|  Copyright (C) 2001 Richard Atterer
  | \/|  <richard@atterer.net>
   ` 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2. See
  the file COPYING for details.

  Interface to the GtkCList of running jobs (downloads etc), window.jobs

*/

#if DEBUG
#  include <iostream>
#endif
namespace std { }
using namespace std;
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include <config.h>
#if WINDOWS
#  include <gdkprivate.h>
#endif

#include <gtk-error.hh>
#include <gtk-support.hh>
#include <joblist.hh>
//______________________________________________________________________

JobVector GUI::jobVector;

gchar* JobVector::noText[] = { 0, 0 }; // For init'ing CList row
vector<GdkPixmap*> JobVector::progressGfx;
//______________________________________________________________________

Job::~Job() { }
//______________________________________________________________________

void JobVector::deleteJob(void* j) {
  delete static_cast<Job*>(j);
}

void JobVector::erase(size_type n) {
  // Delete entry
  Job* j = get(n);
  if (j) {
    Paranoid(entryCountVal > 0);
    --entryCountVal;
    delete j;
    set(n, 0); // 0 => marked for collapse()ation :)
  }
  // Make row blank
  gtk_clist_set_text(list(), n, 0, "");
  gtk_clist_set_text(list(), n, 1, "");
  gtk_clist_set_text(list(), n, 2, "");
  Paranoid(NR_OF_COLUMNS == 3);
}

void collapse2() {
  // TODO
}
//______________________________________________________________________

/* Create progress pixmaps from the big "progress051.xpm" file. The
   reason why we have to resort to pixmaps and not use a
   GtkProgressBar is that GtkCList does not appear to allow you to
   insert an arbitrary widget into a CList, just pixmaps and text.
   Grrr.
   Called from gtk-gui.cc */
void JobVector::initAfterGtk() {
  Assert(GUI::jobVector.list()->columns == 2);
  Assert(GUI::jobVector.empty());
  Paranoid(sizeof(noText) == NR_OF_COLUMNS * sizeof(gchar*));

  GtkWidget* progress = create_pixmap(GUI::window.window, PROGRESS_XPM);
  GdkPixmap* progressImage;
  gint width = -1, height = -1, depth = -1;

  gtk_pixmap_get(GTK_PIXMAP(progress), &progressImage, NULL);
# if WINDOWS
  // This is probably the wrong way to get the width/height...
  width = ((GdkDrawablePrivate*)progressImage)->width;
  height = ((GdkDrawablePrivate*)progressImage)->height;
  depth = 24;
# else
  /* It is not clearly documented whether casting from GdkPixmap* to
     GdkWindow* in this way is OK... :-/ */
  gdk_window_get_geometry((GdkWindow*)progressImage, 0, 0,
                          &width, &height, &depth);
  Assert(width != -1 && height != -1 && depth != -1);
# endif
// # if DEBUG
//   cerr << PROGRESS_XPM << ": " << width << " x " << height << " x "
//        << depth << endl;
// # endif
  GdkGC* gc = gdk_gc_new((GdkWindow*)progressImage);
  Paranoid(gc != 0);
  //____________________

  /* Split up progressImage into the many smaller pixmaps for the
     graphical progress bar. These pixmaps will never be de-allocated
     again. */
  Paranoid(progressGfx.empty());
  progressGfx.reserve(PROGRESS_SUBDIV);
  if (width == 1 && height == 1) {
    // progressGfx not found - fill array with dummy pixmaps
    for (int i = 0; i < PROGRESS_SUBDIV; ++i)
      progressGfx.push_back(progressImage);
  } else {
    int sheight = height / PROGRESS_SUBDIV; // Height of small pixmap
    for (int i = 0; i < height; i += sheight) {
      // Create new pixmap and add to list
      GdkPixmap* pic = gdk_pixmap_new(NULL, width, sheight, depth);
      Paranoid(pic != 0);
      progressGfx.push_back(pic);
      gdk_draw_pixmap((GdkDrawable*)pic, gc, progressImage,
                      0, i, 0, 0, width, sheight);
    }
  }
  //____________________

  gdk_gc_unref(gc);
  gtk_widget_destroy(progress);
}

GdkPixmap* JobVector::progress(uint64 curAmount, uint64 totalAmount) {
  unsigned nr = PROGRESS_SUBDIV * curAmount / totalAmount;
  if (nr >= progressGfx.size()) nr = progressGfx.size() - 1;
  return progressGfx[nr];
}
//______________________________________________________________________

DownloadJob::DownloadJob(const string& uri, const string& dest)
  : download(uri), destination(dest), progressBar(0) { }

/* Note: We do not destroy the widgets, this will be done
   automatically once the row is removed from the GtkCList. */
DownloadJob::~DownloadJob() { }
//______________________________________________________________________

// Create widgets for a download Job
void DownloadJob::run() {
  Paranoid(state() == stopped);



//   errorBox("destdir not writeable");
//   return;

//stat()
//   if (isFilename(destination) || uri.lastChars() != ".jigdo") {
//     justDownload();
//   } else {
//     jigdoDownload();
//   }

  // Show progress info
  gtk_clist_set_pixtext(list(), row(), JobVector::COLUMN_PROGRESS,
                        " 25% done", JobVector::PIXMAP_SPACING,
                        JobVector::progress(GUI::jobVector.size(), 10), 0);

  // Show URL as object name
  gtk_clist_set_text(list(), row(), JobVector::COLUMN_OBJECT,
                     download.uri().c_str());

  setState(running);
}
//________________________________________

// void DownloadJob::dataSize(uint64 n) { }
//void DownloadJob::newData(void* /*data*/, size_t /*size*/, uint64 /*totalSize*/) { }
// void DownloadJob::info(const string& message) { }
// void DownloadJob::error(const string& message) { }
