/* $Id: mimestream.cc,v 1.3 2001/08/22 14:40:47 richard Exp $ -*- C++ -*-
  __   _
  |_) /|  Copyright (C) 2000 Richard Atterer
  | \/|  <richard@atterer.net>
   ` 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License, version 2. See
  the file COPYING for details.

  Convert binary data to/from ASCII using Base64 encoding

*/

#include <config.h>
#include <mimestream.hh>
//______________________________________________________________________

/* The first line (commented out) is the correct string for MIME
   base64 encoding. We use the second instead because b64 strings
   might be used as filenames for jigdo at one point. Additionally,
   "+" or "/" looks weird in the .jigdo file. */
const byte Base64ostream::code[] =
  //"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
  "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-_";

void Base64ostream::close() {
  flush();
  stream = 0;
}

Base64ostream& Base64ostream::put(uint32 x) {
  (*this)
    .put(static_cast<unsigned char>(x & 0xff))
    .put(static_cast<unsigned char>((x >> 8) & 0xff))
    .put(static_cast<unsigned char>((x >> 16) & 0xff))
    .put(static_cast<unsigned char>((x >> 24) & 0xff));
  return *this;
}

Base64ostream& Base64ostream::flush() {
  if (bits > 0) {
    data <<= 6 - bits;
    (*stream) << code[data & 63U];
  }
  bits = 0;
  return *this;
}

Base64ostream& Base64ostream::trailer(streamsize n) {
  int rest = n % 3;
  if (rest == 1)
    (*stream) << '=';
  if (rest >= 1)
    (*stream) << '=';
  return *this;
}

// Output null-terminated string
Base64ostream& Base64ostream::operator<<(const unsigned char* x) {
  while (*x != '\0')
    (*this) << static_cast<byte>(*x++);
  return *this;
}

// Output n characters
Base64ostream& Base64ostream::write(const unsigned char* x, size_t n) {
  for (size_t i = 0; i < n; ++i)
    (*this) << static_cast<byte>(*x++);
  return *this;
}
